package com.xs.core.filter;

import cn.hutool.core.util.ReflectUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.JSONWriter;
import com.alibaba.fastjson2.filter.ContextValueFilter;
import com.xs.core.common.annotation.EntryIgnore;
import com.xs.core.utils.SecurityUtil;
import jakarta.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.MethodParameter;
import org.springframework.core.annotation.Order;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.time.LocalDateTime;

/**
 * 响应加解密拦截器
 */
@Component
@ControllerAdvice
@Order(value = 20)
public class ResponseHandler implements ResponseBodyAdvice<Object> {

    @Value("${encryption.isEncryption}")
    private boolean isEncryption;

    @Value("${encryption.responsePublicKey}")
    private String responsePublicKey;

    /**
     * 返回true，才会走beforeBodyWrite方法
     */
    @Override
    public boolean supports(MethodParameter methodParameter, Class<? extends HttpMessageConverter<?>> aClass) {
        Method method = methodParameter.getMethod();
        //如过方法上有注解，直接返回false，不走beforeBodyWrite方法
        if (method != null && method.isAnnotationPresent(EntryIgnore.class)) {
            return false;
        }
        return isEncryption;
    }

    /**
     * 响应加密
     */
    @Override
    public Object beforeBodyWrite(Object body, MethodParameter methodParameter, MediaType mediaType, Class<? extends HttpMessageConverter<?>> aClass, ServerHttpRequest request, ServerHttpResponse serverHttpResponse) {
        if (isEncryption) {
            ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
            HttpServletRequest httpRequest = requestAttributes.getRequest();
            if (StringUtils.isNotBlank(httpRequest.getHeader("Knife4j-Gateway-Code"))) {
                return body;
            }
//            JSON.configWriterDateFormat(DatePattern.NORM_DATETIME_PATTERN);
            // 拿到响应的数据
            String json = JSON.toJSONString(body, (ContextValueFilter) (context, object, name, value) -> {
                        Field field = ReflectUtil.getField(object.getClass(), name);
                        //处理日期类型为空时返回空字符串
                        if (null != field && value == null && LocalDateTime.class.isAssignableFrom(field.getType())) {
                            return "";
                        }
                        return value;
                    },
                    JSONWriter.Feature.PrettyFormat,
                    JSONWriter.Feature.WriteNullStringAsEmpty,
                    JSONWriter.Feature.WriteNullListAsEmpty,
                    JSONWriter.Feature.WriteMapNullValue,
                    JSONWriter.Feature.WriteNulls);
            // 进行加密
            String result = SecurityUtil.encryptSM2(responsePublicKey, json);
            String sign = SecurityUtil.getSign(result);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("encryptData", result);
            jsonObject.put("identifying", sign);
            return jsonObject;
        }
        return body;
    }
}