package com.xs.core.mq.consumer;

import cn.hutool.core.util.ObjUtil;
import com.alibaba.fastjson2.JSON;
import com.rabbitmq.client.Channel;
import com.xs.core.common.constant.GoldCoinConstant;
import com.xs.core.config.mq.RabbitMQConfig;
import com.xs.core.model.coin.dto.GoldRateUpgradesState;
import com.xs.core.model.coin.msg.CoinProducerMessage;
import com.xs.core.model.coin.entity.GoldCoinProdRecord;
import com.xs.core.model.coin.entity.GoldCoinProdState;
import com.xs.core.mq.producer.GoldCoinMessageProducer;
import com.xs.core.service.coin.IGoldCoinProdRecordService;
import com.xs.core.service.coin.IGoldCoinProdStateService;
import com.xs.core.service.redis.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;

@Component
@Slf4j
public class GoldCoinProductMessageConsumer {
    @Autowired
    private RedisService redisService;

    @Autowired
    private GoldCoinMessageProducer goldCoinProducer;

    @RabbitListener(queues = RabbitMQConfig.GOLD_COIN_CALCULATION_QUEUE, concurrency = "10")
    public void handleCalculation(Message m, Channel channel) throws IOException {
        CoinProducerMessage message = null;
        try {
            String msg = new String(m.getBody());
            message = JSON.parseObject(msg, CoinProducerMessage.class);
            // 处理计算逻辑
            if (shouldContinueCalculation(message)) {
                GoldCoinProdState coinProdState = calculateAndUpdateGame(message);
                if (coinProdState != null) {
                    // 更新Redis中的金币产出状态
                    redisService.set(GoldCoinConstant.GOLD_COIN_STATE_KEY + message.getUserId(), coinProdState);
                    // 发送下一次延迟计算消息
                    if (!coinProdState.isCompleted()) {
                        CoinProducerMessage newMessage = new CoinProducerMessage();
                        BeanUtils.copyProperties(coinProdState, newMessage);
                        goldCoinProducer.sendDelayCalculationMessage(newMessage);
                    } else {
                        //发送金币结算消息
                        log.info("Send gold coin complete message for user: {}", message.getUserId());
                        CoinProducerMessage newMessage = new CoinProducerMessage();
                        BeanUtils.copyProperties(coinProdState, newMessage);
                        goldCoinProducer.sendCalculationCompleteMessage(newMessage);
                    }
                } else {
                    // 记录错误日志，可以考虑报警
                    log.error("Failed to calculate for user: {}", message.getUserId());
                }
            }
            channel.basicAck(m.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            log.error("Failed to handle calculation for user: " + message.getUserId(), e);
            handleCalculationError(message);
        }
    }

    private GoldCoinProdState calculateAndUpdateGame(CoinProducerMessage message) {
        String key = GoldCoinConstant.GOLD_COIN_STATE_KEY + message.getUserId();
        //拿到当前的金币产出状态
        GoldCoinProdState goldCoinProdState = redisService.get(key, GoldCoinProdState.class);
        if (ObjUtil.isNotNull(goldCoinProdState)) {
            // 计算金币产出
            return calculateGoldCoin(goldCoinProdState);
        } else {
            // 记录错误日志，可以考虑报警
            log.error("Failed to get gold coin prod state for user: {}", message.getUserId());
            return null;
        }
    }

    /**
     * 计算金币产出
     *
     * @param goldCoinProdState
     */
    private GoldCoinProdState calculateGoldCoin(GoldCoinProdState goldCoinProdState) {
        log.info("计算金币产出======>{}", goldCoinProdState.getUserId());
        String rateUpgradesKey = GoldCoinConstant.GOLD_COIN_RATE_UPGRADES_KEY + goldCoinProdState.getUserId();
        //拿到速率升级状态
        GoldRateUpgradesState upgradesState = redisService.get(rateUpgradesKey, GoldRateUpgradesState.class);
        int rateupdateFLag = 0;
        if (ObjUtil.isNotNull(upgradesState)) {
            rateupdateFLag = upgradesState.getRateUpdateFlag();
        }
        long currentTimestamp = Instant.now().getEpochSecond();
        Long lastTimestamp = goldCoinProdState.getLastTimestamp();
        Long endTimestamp = goldCoinProdState.getEndTimestamp();
        long timeDifference;
        boolean isCompleted = false;

        // 判断是否超过结束时间
        if (endTimestamp != null && currentTimestamp >= endTimestamp) {
            // 如果超过结束时间，只计算到结束时间为止
            timeDifference = endTimestamp - lastTimestamp;
            isCompleted = true;
        } else {
            timeDifference = currentTimestamp - lastTimestamp;
        }

        // 如果时间差为 0 或负数，无需更新
        if (timeDifference <= 0) {
            // 如果时间差为 0 或负数，直接返回
            goldCoinProdState.setCompleted(true);
            return goldCoinProdState;
        }

        BigDecimal currentValue = goldCoinProdState.getCurrentValue();
        BigDecimal newValue;

        //若是速率升级
        if (rateupdateFLag == 1) {
            Long rateUpdateTimestamp = upgradesState.getUpdateTimestamp();

            // 安全检查：确保 rateUpdateTimestamp 在有效范围内
            if (rateUpdateTimestamp == null || rateUpdateTimestamp < lastTimestamp || rateUpdateTimestamp > (isCompleted ? endTimestamp : currentTimestamp)) {
                rateUpdateTimestamp = isCompleted ? endTimestamp : currentTimestamp;
            }

            // 计算旧速率的金币产出
            long oldRateDuration = Math.min(rateUpdateTimestamp - lastTimestamp, timeDifference);
            BigDecimal oldRate = new BigDecimal(goldCoinProdState.getCurrentRate());
            newValue = currentValue.add(oldRate.multiply(BigDecimal.valueOf(oldRateDuration)));

            // 计算新速率的金币产出
            long newRateDuration = timeDifference - oldRateDuration;
            String newRateStr = upgradesState.getCurrentRate();
            //时间差大于0，再计算新速率的值
            if (newRateDuration > 0) {
                BigDecimal newRate = new BigDecimal(newRateStr != null ? newRateStr : "0");
                newValue = newValue.add(newRate.multiply(BigDecimal.valueOf(newRateDuration)));
            }
            // 处理速率升级的状态
            goldCoinProdState.setCurrentRate(newRateStr);
            goldCoinProdState.setDurableRate(upgradesState.getDurableRate());
            goldCoinProdState.setTemporaryRate(upgradesState.getTemporaryRate());
            upgradesState.setRateUpdateFlag(2);
            //处理临时速率的状态
            if (upgradesState.getBoostRateFLag() == 0 && upgradesState.isHasTemporaryRate()) {
                upgradesState.setBoostRateFLag(1);
                goldCoinProdState.setBoostResidueTimestamp(GoldCoinConstant.BOOST_RATE_RESIDUE_TIMESTAMP);
            }
            redisService.set(rateUpgradesKey, upgradesState);
        } else {
            // 使用当前速率计算
            BigDecimal currentRate = new BigDecimal(goldCoinProdState.getCurrentRate());
            newValue = currentValue.add(currentRate.multiply(BigDecimal.valueOf(timeDifference)));
        }
        // 更新通用状态
        goldCoinProdState.setLastTimestamp(isCompleted ? endTimestamp : currentTimestamp);
        goldCoinProdState.setCurrentValue(newValue);
        goldCoinProdState.setResidueTimestamp(Math.max(0, goldCoinProdState.getResidueTimestamp() - timeDifference));
        goldCoinProdState.setBoostResidueTimestamp(Math.max(0, (ObjUtil.isNotNull(goldCoinProdState.getBoostResidueTimestamp()) ? goldCoinProdState.getBoostResidueTimestamp() : 0L) - timeDifference));
        // 如果达到或超过结束时间，设置相应的标志
        if (isCompleted) {
            goldCoinProdState.setCompleted(true);
        }
        return goldCoinProdState;
    }


    /**
     * 判断是否需要继续计算
     *
     * @param message
     * @return
     */
    private boolean shouldContinueCalculation(CoinProducerMessage message) {
        // 处理判断是否需要计算的逻辑
        return message.getRunning() && message.getResidueTimestamp() > 0;
    }

    private void handleCalculationError(CoinProducerMessage message) {
        if (message.getRetryCount() < 3) {
            message.setRetryCount(message.getRetryCount() + 1);
            goldCoinProducer.sendDelayCalculationMessage(message);
        } else {
            // 记录错误日志，可以考虑报警
            log.error("Failed to calculate after 3 retries for user: {}", message.getUserId());
        }
    }

}
