package com.xs.core.mq.producer;

import com.alibaba.fastjson2.JSON;
import com.xs.core.config.mq.RabbitMQConfig;
import com.xs.core.model.coin.msg.CoinProducerMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class GoldCoinMessageProducer {
    @Autowired
    private RabbitTemplate rabbitTemplate;

    /**
     * 发送开始游戏的消息
     */
    public void sendDelayCalculationMessage(CoinProducerMessage coinProducerMessage) {
        try {
            String messageStr = JSON.toJSONString(coinProducerMessage);
            rabbitTemplate.convertAndSend(RabbitMQConfig.GOLD_COIN_CALCULATION_EXCHANGE, RabbitMQConfig.GOLD_COIN_CALCULATION_QUEUE, messageStr, msg -> {
                //延时10秒
                msg.getMessageProperties().setHeader("x-delay", 10 * 1000L);
                return msg;
            });
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            // 重试机制
            handleSendFailure(coinProducerMessage);
        }
    }

    private void handleSendFailure(CoinProducerMessage message) {
        if (message.getRetryCount() < 3) {
            message.setRetryCount(message.getRetryCount() + 1);
            // 使用指数退避策略
            int delay = (int) Math.pow(2, message.getRetryCount()) * 1000;
            // 可以使用线程池来处理重试
            CompletableFuture.delayedExecutor(delay, TimeUnit.MILLISECONDS)
                    .execute(() -> sendDelayCalculationMessage(message));
        } else {
            // 记录失败日志，可以考虑报警
            log.error("Failed to send message after 3 retries for user: {}", message.getUserId());
        }
    }

    /**
     * 发送计算完成的消息
     *
     * @param coinProducerMessage
     */
    public void sendCalculationCompleteMessage(CoinProducerMessage coinProducerMessage) {
        try {
            String messageStr = JSON.toJSONString(coinProducerMessage);
            rabbitTemplate.convertAndSend(RabbitMQConfig.GOLD_COIN_COMPLETE_EXCHANGE, RabbitMQConfig.GOLD_COIN_COMPLETE_QUEUE, messageStr);
            log.info("发送计算结束的消息: {}", messageStr);
        } catch (Exception e) {
            e.printStackTrace();
            // 重试机制
            handleSendFailure(coinProducerMessage);
        }
    }
}
