package com.xs.core.utils.redis;

import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class RedissonLockUtil {
    public static RedissonLockUtil lockUtil;
    private final RedissonClient redissonClient;

    private static final Long PRO_LOCK_RELEASE_TIME = 65L;//释放锁时间
    private static final Long PROC_LOCK_WAIT_TIME = 2L;//获取锁等待时间

    public RedissonLockUtil(RedissonClient redissonClient) {
        this.redissonClient = redissonClient;
        lockUtil = this;
    }

    /**
     * 根据name对进行上锁操作，redisson Lock 一直等待获取锁
     */
    public static void lock(String lockName, long time) {
        RLock lock = lockUtil.redissonClient.getLock(lockName);
        log.info("lock.isLocked:{}", lock.isLocked());
        //lock提供带timeout参数，timeout结束强制解锁，防止死锁
        lock.lock(time, TimeUnit.SECONDS);
    }


    /**
     * 根据name对进行上锁操作，redisson tryLock  根据第一个参数，一定时间(waitTime)内未获取到锁，则不再等待直接返回boolean
     *
     * @param lockName  锁名
     * @param waitTime  等待获取锁时间
     * @param leaseTime 上锁后释放锁时间
     */
    public static boolean tryLock(String lockName, long waitTime, long leaseTime) throws InterruptedException {
        RLock lock = lockUtil.redissonClient.getLock(lockName);
        //tryLock，第一个参数是等待时间，waitTime内获取不到锁，则直接返回。 第二个参数 释放锁时间
        return lock.tryLock(waitTime, leaseTime, TimeUnit.SECONDS);
    }

    /**
     * 尝试获取锁 默认等待2秒  65秒自动释放锁
     *
     * @param lockName 锁名称
     * @return
     * @throws InterruptedException
     */
    public static boolean tryLock(String lockName) throws InterruptedException {
        RLock lock = lockUtil.redissonClient.getLock(lockName);
        //tryLock，第一个参数是等待时间，waitTime内获取不到锁，则直接返回。 第二个参数 释放锁时间
        return lock.tryLock(PROC_LOCK_WAIT_TIME, PRO_LOCK_RELEASE_TIME, TimeUnit.SECONDS);
    }

    /**
     * 释放锁操作
     */
    public static void unLock(String lockName) {
        RLock lock = lockUtil.redissonClient.getLock(lockName);
        lock.forceUnlock();
    }
}
